# encoding: UTF-8
# frozen_string_literal: true

require 'nrser/refinements/types'
using NRSER::Types


# Namespace
# =======================================================================

module  Locd
module  CLI
module  Command


# Definitions
# =======================================================================

class Job < Agent
  
  desc              "add [OPTIONS] -- CMD_TEMPLATE...",
                    "Add job that runs in the current directory"
  
  include_shared    t[  groups:
                          t.has_any(  :write,
                                      :add,
                                      :respond_with_agents  ) ]

  option            :every,
                    desc: "How often to start the job",
                    type: :string
  
  option            :at,
                    desc: "Day/time to start the job",
                    type: :hash
  
  def add           *cmd_template, **kwds
    logger.trace "#{ self.class.name }##{ __method__ }",
      options: options,
      cmd_template: cmd_template,
      kwds: kwds,
      shared: self.class.shared_method_options
    
    kwds[:start_interval] ||= get_start_interval
    
    super *cmd_template, **kwds
  end

end # class Job


# /Namespace
# =======================================================================

end # module  Command
end # module  CLI
end # module  Locd
